@@ -1,4 +1,3 @@
-import deepdish as dd
 import sys
 import numpy as np
 from scipy.sparse.linalg import lsqr 
@@ -6,19 +5,22 @@ from scipy.sparse.linalg import inv as sinv
 from scipy.linalg import inv as inv
 from scipy.sparse import csc_matrix
 import scipy
-import deepdish as dd
-
+from .utils import *
+#import deepdish as dd
 
 
 
 
 def energy_conserving(W):
 
+
  bottom = np.sum(np.absolute(W))
  r = np.sum(np.absolute(np.einsum('ij->j',W)))
- #print('Row check:' + str(r/bottom))
+ print('Row check:' + str(r/bottom))
+ 
  c = np.sum(np.absolute(np.einsum('ij->i',W)))
- #print('Column Check:' + str(c/bottom))
+ print('Column Check:' + str(c/bottom))
+ 
  nm = np.shape(W)[0]
  delta = np.einsum('ij->j',W)
  b = -2*np.concatenate((delta,delta))
@@ -44,15 +46,17 @@ def energy_conserving(W):
  c = np.sum(np.absolute(np.einsum('ij->i',W)))
  print('Column Check:' + str(c/bottom))
 
+ return W
 
 def generate_full(**argv):
 
- filename = 'full.h5'
-
+ filename = 'full.npz'
 
  print(' ')
  print('Importing ' + filename + ' ... ',end= '')
- data = dd.io.load(filename)
+ #data = np.load(filename,allow_pickle=True)
+ data = load_data('full')
+ #data = dd.io.load(filename)
  print(' done')
  print(' ')
 
@@ -70,14 +74,28 @@ def generate_full(**argv):
  nm = W.shape[0]
 
  print('Making W energy conserving ...',end= '')
- energy_conserving(W)
+ W = energy_conserving(W)
  print('... done')
  print(' ')
  print('Computing kappa bulk ...')
- kappa = np.einsum('ui,uq,qj->ij',sigma,np.linalg.pinv(W),sigma)
+ kappa = np.einsum('ui,uq,qj->ij',sigma,np.linalg.pinv(W),sigma)/data['alpha']
  print(kappa)
  print(' done')
  print(' ')
+ #----------
+ #test
+ #Adinv = np.diag(1/np.diag(W))
+ #Aod = W - np.diag(np.diag(W))
+ #Add = np.einsum('ij,i->ij',Aod,1/np.diag(W))
+ #x0 = np.einsum('lk,kj->lj',Adinv,sigma)
+ #x_old = np.zeros_like(x0)
+ #for n in range(40):
+ #    print(n) 
+ #    x = x0 - np.einsum('kl,lj->kj',Add,x_old)
+ #    print(np.einsum('li,lj->ij',sigma,x)/data['alpha'])
+ #    x_old = x.copy()
+ #quit()
+ #----------
 
  #postprocessing----
  B = -np.einsum('i,ij->ij',1/np.diag(W),W-np.diag(np.diag(W)))
@@ -91,7 +109,7 @@ def generate_full(**argv):
 
  F = np.einsum('i,ij->ij',1/np.diag(W),sigma)
 
- data = {'tc':tc,'VMFP':F,'sigma':sigma,'kappa':kappa,'B':B,'scale':1/np.diag(W)}
+ data = {'tc':tc,'VMFP':F[:,:2],'sigma':sigma[:,:2],'kappa':kappa,'B':B,'scale':1/np.diag(W),'model':[10],'alpha':data['alpha']}
 
  return data
 
