@@ -5,15 +5,14 @@ from .mesher import *
 from .utils import *
 import time
 import scipy.sparse as sp
-import deepdish as dd
 import itertools
-from .GenerateInterface import *
 from mpi4py import MPI
 import time
-#from matplotlib.pylab import *
 import numpy.testing as npt
 from statistics import mean
-#from matplotlib.pylab import *
+from scipy.ndimage.interpolation import shift
+from collections import Counter
+
 
 comm = MPI.COMM_WORLD
 
@@ -22,25 +21,19 @@ class Geometry(object):
  def __init__(self,**argv):
 
   if comm.rank == 0:
-   if argv.setdefault('model','lattice') == 'interface':
-     GenerateInterface(**argv) 
+   if argv.setdefault('user',False):
+     self.import_mesh(**argv)
+     argv.update({'centroids':self.elem_centroids})
+     self.elem_mat_map = argv['correlation'](**argv)
    else:
      Mesher(argv) #this create mesh.msh
      self.dmin = argv['dmin']
      self.import_mesh(**argv)
-     self.data = self.compute_mesh_data(**argv)
-     if argv.setdefault('save',True):
-      a = time.time()
-      #np.save('geometry',self.data)
-      dd.io.save('geometry.h5',self.data)
-
-
- def compute_node_map(self,**argv):
-
-   self.conn = np.zeros(len(self.nodes))
-   for k,e in enumerate(self.elems):
-     for n in e:
-      self.conn[n] +=1
+     self.elem_mat_map = np.zeros(len(self.elems)) #omhogeneous material
+   self.update_side_interface()
+   self.data = self.compute_mesh_data(**argv)
+   if argv.setdefault('save',True):
+    np.savez_compressed('geometry',self.data)   
 
 
  def compute_boundary_connection(self):
@@ -54,6 +47,16 @@ class Geometry(object):
         d1 = normal * np.dot(normal,d)
         self.bconn.append(d1)
 
+ def update_side_interface(self):
+
+  interface_sides = []
+  for ll in self.side_list['active']:
+     (k1,k2) = self.side_elem_map[ll]
+     if not self.elem_mat_map[k1] == self.elem_mat_map[k2]:
+             interface_sides.append(ll)
+  interface_sides = np.array(interface_sides)
+  self.side_list['Interface'] = interface_sides  
+
 
  def compute_mesh_data(self,**argv):
 
@@ -66,35 +69,49 @@ class Geometry(object):
     self.compute_interpolation_weigths()
     self.compute_dists()
     self.compute_boundary_condition_data(argv)
-    self.compute_node_map()
+    #self.compute_node_map()
 
     self.n_elems = len(self.elems)
 
+    if self.dim == 3:
+     filling = np.sum(self.elem_volumes)/self.size[0]/self.size[1]/self.size[2]
+    else: 
+     filling = np.sum(self.elem_volumes)/self.size[0]/self.size[1]
+    self.computed_porosity = 1-filling
+
+
     #generate_elem_mat_map
 
     #generate_frame
     frame = generate_frame(**argv)
     #-----------
-    #argv.update({'centroids':self.elem_centroids})
-    #self.elem_mat_map = CreateCorrelation(**argv)
-    self.elem_mat_map = { i:[0] for i in range(len(self.elems))}
 
     periodic_side_values_vec = []
     for ll in self.side_list['Periodic']:
       periodic_side_values_vec.append(self.periodic_side_values[ll])
       
-    weigths_vec = np.zeros((self.n_elems,self.dim,len(self.elems[0])))  
-    for value,items in self.weigths.items(): 
-     weigths_vec[value] = items
+    weigths_vec = np.zeros((self.n_elems,self.dim,max(self.side_per_elem)))  
+    for value,items in self.weigths.items():
+        weigths_vec[value,:,:np.shape(items)[1]] = items
 
     interp_weigths = np.zeros(len(self.sides))
     for value,items in self.interp_weigths.items():
       interp_weigths[value] = items[0]
-    
+   
+    #Fill with -1 the missing connectivity
+    ms = max(self.side_per_elem)
+    for n,e in enumerate(self.elems):
+        if len(e) < ms:
+            e.append(-1)
+            self.elem_side_map[n].append(-1)
+
+
+
     return {
           'size':self.size,\
-          'conn':self.conn,\
-          'elems':self.elems,\
+          #'conn':self.conn,\
+          'elems':np.array(self.elems),\
+          'boundary_element': np.array([self.side_elem_map[ll][0] for ll in self.side_list['Boundary']],dtype=np.int),\
           'dim':np.array([self.dim]),\
           'weigths': weigths_vec,\
           'bconn': np.array(self.bconn),\
@@ -109,12 +126,16 @@ class Geometry(object):
           'centroids':np.array(self.elem_centroids),\
           'side_centroids':np.array(self.side_centroids),\
           'volumes':self.elem_volumes,\
-          'kappa_mask':np.array(np.sum(self.B_with_area_old.todense(),axis=0))[0],\
+          'elem_mat_map':self.elem_mat_map,\
+          'kappa_mask':-np.array(np.sum(self.B_with_area_old.todense(),axis=0))[0]*self.kappa_factor*1e-18,\
           'dists':np.array(self.dists_side),\
           'flux_sides':np.array(self.flux_sides),\
           'frame':frame,\
           'i':np.array(self.i),\
           'j':np.array(self.j),\
+          'inti':np.array(self.inti),\
+          'intj':np.array(self.intj),\
+          'intk':np.array(self.intk),\
           'im': np.concatenate((self.i,list(np.arange(self.n_elems)))),\
           'jm': np.concatenate((self.j,list(np.arange(self.n_elems)))),\
           'k':self.k,\
@@ -123,10 +144,13 @@ class Geometry(object):
           'db':self.db,\
           'active_sides':np.array(self.side_list['active']),
           'inactive_sides':np.array(self.side_list['Inactive']),
+          'interface_sides':np.array(self.side_list['Interface']),
           'periodic_sides':np.array(self.side_list['Periodic']),
           'boundary_sides':np.array(self.side_list['Boundary']),
           'n_side_per_elem': np.array([len(i)  for i in self.elems]),
+          'n_non_boundary_side_per_elem': np.array(self.n_non_boundary_side_per_elem,dtype=np.int),
           'pp':np.array(self.pp),\
+          'side_per_elem':np.array(self.side_per_elem),\
           'meta':np.asarray([self.n_elems,self.kappa_factor,self.dim,len(self.nodes),len(self.side_list['active'])],np.float64)}
 
 
@@ -153,7 +177,8 @@ class Geometry(object):
    nc = len(self.elems)
    ns = len(self.elem_side_map[0])
    self.i = [];self.j = [];self.k = []
-   self.eb = [];self.sb = []; self.db = []; self.dbp = []
+   self.eb = [];self.sb = []; self.db = []; #self.dbp = []
+   self.inti = [];self.intj = []; self.intk = []; 
 
    data = []
  
@@ -163,25 +188,39 @@ class Geometry(object):
      vol1 = self.elem_volumes[l1]
      vol2 = self.elem_volumes[l2]
      area = self.side_areas[ll]
-     normal = self.face_normals[ll]
-     if not l1 == l2:
-      self.i.append(l1)   
-      self.j.append(l2)   
-      self.i.append(l2)   
-      self.j.append(l1)   
-      ij.append([l1,l2])
-      ij.append([l2,l1])
-      self.k.append(normal*area/vol1)
-      self.k.append(-normal*area/vol2)
+     normal = self.face_normals[ll][:self.dim]
+
+     if ll in self.side_list['Interface']:
+      self.inti.append(l1)   
+      self.intj.append(l2)   
+      self.inti.append(l2)   
+      self.intj.append(l1)   
+      self.intk.append(normal*area/vol1)
+      self.intk.append(-normal*area/vol2)
+      #ij.append([l1,l2])
+      #ij.append([l2,l1])
+
+     if not l1 == l2 :#and not (ll in self.side_list['Interface']):
+       self.i.append(l1)   
+       self.j.append(l2)   
+       self.i.append(l2)   
+       self.j.append(l1)   
+       ij.append([l1,l2])
+       ij.append([l2,l1])
+       self.k.append(normal*area/vol1)
+       self.k.append(-normal*area/vol2)
      else:
-       #s = np.where(np.array(self.elem_side_map[l1])==ll)[0][0]
        self.eb.append(l1)
        self.sb.append(ll)
        self.db.append(normal*area/vol1)
    
    self.k = np.array(self.k).T
    self.db = np.array(self.db).T
+   self.intk = np.array(self.intk).T
 
+   
+   #self.i += self.inti
+   #self.j += self.intj
    self.ij = ij
 
 
@@ -208,7 +247,6 @@ class Geometry(object):
    lines = [l.split()  for l in lines]
    nb = int(lines[4][0])
    current_line = 5
-   #Get physical regions
    self.blabels = {int(lines[current_line+i][1]) : lines[current_line+i][2].replace('"',r'') for i in range(nb)}
 
    current_line += nb+2
@@ -216,13 +254,10 @@ class Geometry(object):
    current_line += 1
    self.nodes = np.array([lines[current_line + n][1:4] for n in range(n_nodes)],float)
    current_line += n_nodes+1 
-   #Get size
    self.size = [ np.max(self.nodes[:,i]) - np.min(self.nodes[:,i])   for i in range(3)]
    self.dim = 2 if self.size[2] == 0 else 3
    current_line += 1
-   #load elements----
    n_elem_tot = int(lines[current_line][0])
-
    current_line += 1
 
    #type of elements
@@ -232,9 +267,8 @@ class Geometry(object):
 
    bulk_tags = [n for n in range(n_elem_tot) if int(lines[current_line + n][1]) in bulk_type[self.dim]] 
    face_tags = [n for n in range(n_elem_tot) if int(lines[current_line + n][1]) in face_type[self.dim]]
-
-   self.elems = [list(np.array(lines[current_line + n][5:],dtype=int)-1) for n in bulk_tags] 
-
+   self.elems = [list(np.array(lines[current_line + n][5:],dtype=int)-1) for n in bulk_tags]
+   self.side_per_elem = np.array([len(e) for e in self.elems])
    self.n_elems = len(self.elems)
    boundary_sides = np.array([ sorted(np.array(lines[current_line + n][5:],dtype=int)) for n in face_tags] ) -1
 
@@ -354,9 +388,25 @@ class Geometry(object):
    for side in self.side_list['Boundary'] :# + self.side_list['Hot'] + self.side_list['Cold']:
     self.side_elem_map[side].append(self.side_elem_map[side][0])
 
-   self.elem_kappa_map = {}
-   self.elem_mat_map = { ne:0 for ne in list(range(len(self.elems)))}
+   #Put boundaries at the end
 
+
+   self.n_non_boundary_side_per_elem = []
+   for key,value in self.elem_side_map.items():
+       tmp_1 = []
+       tmp_2 = []
+       for s in value:
+         if s in self.side_list['Boundary']:
+           tmp_1.append(s)
+         else:  
+           tmp_2.append(s)
+       self.n_non_boundary_side_per_elem.append(len(tmp_2)+len(tmp_1))
+       #self.n_non_boundary_side_per_elem.append(len(tmp_2))
+       self.elem_side_map[key] = tmp_2 + tmp_1
+
+   #------------------------
+
+   
     
 
  def compute_face_normal(self,ns):
@@ -396,8 +446,17 @@ class Geometry(object):
 
    M = np.ones((3,3))
    for i,e in enumerate(self.elems):
-    M[:,:2] = self.nodes[e,0:2]
-    self.elem_volumes[i] = abs(0.5*np.linalg.det(M))
+    if len(e) == 3:     
+     M[:,:2] = self.nodes[e,0:2]
+     self.elem_volumes[i] = abs(0.5*np.linalg.det(M))
+    else:
+
+     points = self.nodes[e]
+     x = points[:,0]
+     y = points[:,1]
+     self.elem_volumes[i] = 0.5*np.abs(np.dot(x,np.roll(y,1))-np.dot(y,np.roll(x,1)))
+
+
 
 
 
@@ -554,6 +613,7 @@ class Geometry(object):
 
       if abs(side_value[side[1]]) > 0:
        self.pp += ((self.ij.index([j,i]),side_value[side[1]]),)
+       #self.pp.append(self.ij.index([j,i]))
        self.ip.append(j); self.jp.append(i); self.dp.append(side_value[side[1]]); self.pv.append(-normal*area/volj)
 
       if np.linalg.norm(np.cross(self.face_normals[side[0]],applied_grad)) < 1e-12:
@@ -568,14 +628,14 @@ class Geometry(object):
      normal = self.face_normals[ll]   
      tmp = np.abs(np.dot(normal,flux_dir))
      if tmp > delta : #either negative or positive
-       if normal[0] == 1:
+       if abs(normal[0]) == 1:
            if self.dim == 2:  
             area_flux = self.size[1]
            else: 
             area_flux = self.size[1]*self.size[2]
            total_area += self.side_areas[ll]
 
-       elif normal[1] == 1:
+       elif abs(normal[1]) == 1:
            if self.dim == 2:  
             area_flux = self.size[0]
            else: 
@@ -595,6 +655,8 @@ class Geometry(object):
 
    nd = len(self.elems[0])
    diff_dist = {}
+   #fig = figure()
+   #ax = fig.add_subplot(111, projection='3d')
 
    for ll in self.side_list['active'] :
     elems = self.side_elem_map[ll]
@@ -607,17 +669,25 @@ class Geometry(object):
      kc2 = elems[1]
      c2 = self.get_next_elem_centroid(kc1,ll)
      dist = c2-c1
-
      ind2 = self.elem_side_map[kc2].index(ll)
-     for i in range(self.dim):
-      diff_dist.setdefault(kc1,np.zeros((len(self.elem_side_map[kc1]),self.dim)))[ind1][i] = dist[i]   
-      diff_dist.setdefault(kc2,np.zeros((len(self.elem_side_map[kc2]),self.dim)))[ind2][i] = -dist[i]   
+     diff_dist.setdefault(kc1,np.zeros((len(self.elem_side_map[kc1]),self.dim)))[ind1] =  dist[:self.dim]
+     diff_dist.setdefault(kc2,np.zeros((len(self.elem_side_map[kc2]),self.dim)))[ind2] = -dist[:self.dim]
+       
+     #diff_dist.setdefault(kc1,np.zeros((self.n_non_boundary_side_per_elem[kc1],self.dim)))[ind1] =  dist
+     #diff_dist.setdefault(kc2,np.zeros((self.n_non_boundary_side_per_elem[kc2],self.dim)))[ind2] = -dist 
+
+
+     #if kc1 == 591:   
+     # ax.scatter(c1[0],c1[1],c1[2],color='k')
+     # ax.scatter(c2[0],c2[1],c2[2],color='k')
+     #if kc2 == 591:   
+     # ax.scatter(c1[0],c1[1],c1[2],color='k')
+     # ax.scatter(c2[0],c2[1],c2[2],color='k')
+         
+
     else :
      dist = self.side_centroids[ll] - c1
-    # kk = list(self.side_list['Boundary']).index(ll)   
-    # dist = self.bconn[kk]
-     for i in range(self.dim):
-      diff_dist.setdefault(kc1,np.zeros((len(self.elem_side_map[kc1]),self.dim)))[ind1][i] = dist[i] 
+     diff_dist.setdefault(kc1,np.zeros((len(self.elem_side_map[kc1]),self.dim)))[ind1] = dist[:self.dim] 
       
      #if ll in self.side_list['Interface']:
      # kc2 = elems[1]
@@ -627,11 +697,27 @@ class Geometry(object):
      # for i in range(self.dim):
      #   diff_dist.setdefault(np.zeros((len(self.elem_side_map[kc2]),self.dim)))[ind2][i] = dist[i]   
 
+   #print(diff_dist)
+   #quit()
+
+
    #Compute weights
    self.weigths = {}
    for h in diff_dist.keys() :
-    tmp = diff_dist[h]   
-    self.weigths[h] = np.dot(np.linalg.inv(np.dot(np.transpose(tmp),tmp)),np.transpose(tmp)  )
+   
+    #if h == 591:  
+    #     nodes = self.nodes[self.elems[h]]
+    #     for n in nodes:
+    #      ax.scatter(n[0],n[1],n[2])
+    #     show()
+
+    tmp = diff_dist[h]
+    #a =  np.dot(np.linalg.pinv(np.dot(np.transpose(tmp),tmp)),np.transpose(tmp))
+    #b = np.linalg.pinv(tmp)
+    #print(np.allclose(a,b))
+    #print(np.dot(np.transpose(tmp),tmp))
+    #self.weigths[h] = np.dot(np.linalg.inv(np.dot(np.transpose(tmp),tmp)),np.transpose(tmp))
+    self.weigths[h] = np.linalg.pinv(tmp)
 
 
 
