@@ -9,41 +9,46 @@ from .utils import *
 import subprocess
 import os
 import time
-#import matplotlib.pylab as plt
+import matplotlib.pylab as plt
 
 class Mesher(object):
 
  def __init__(self,argv):
-  argv['base'] = np.array(argv['base'])
 
-  self.add_symmetry(argv)
+  argv['dmin'] = 0
+  model = argv.setdefault('model','lattice')
 
-  if argv.setdefault('model','lattice') == 'lattice':   
+  if model == 'lattice':   
    #create polygons-----
-   base = argv.setdefault('base',[[0,0]])
-   if len(base) == 0 :
-    argv['dmin'] = 1e5
+   self.add_symmetry(argv) 
+   shapes = get_shape(argv)
+   polygons = [translate_shape(shapes[n],i) for n,i in enumerate(argv['base'])]
+   argv.update({'polygons':np.array(polygons)})
+   repeat_merge_scale(argv)
+
+  elif model == 'custom':
+    repeat_merge_scale(argv)
+
+  elif model =='bulk':
+
     if argv.setdefault('lz',0) == 0:
      return self.generate_bulk_2D(argv)
     else :
      return self.generate_bulk_3D(argv)
 
-   a = time.time()
-   shapes = get_shape(argv)
+  elif model == 'random':
+      while argv['dmin'] < argv.setdefault('dmin_prescribed',0.02):  
+       argv['base'] = np.random.rand(argv.setdefault('Np',5),2)-0.5 
 
-
-   polygons = [translate_shape(shapes[n],i) for n,i in enumerate(base)]
-
-
-   argv.update({'polygons':np.array(polygons)})
-
-  repeat_merge_scale(argv)
+       self.add_symmetry(argv) 
+       self.add_polygons(argv)
+       repeat_merge_scale(argv)
   #---------------------
 
   if argv.setdefault('lz',0) == 0:
-     self.generate_mesh_2D(argv)
+    self.generate_mesh_2D(argv)
   else:   
-     self.generate_mesh_3D(argv)
+    self.generate_mesh_3D(argv)
 
 
  def generate_bulk_3D(self,argv):
@@ -196,13 +201,21 @@ class Mesher(object):
     output = subprocess.check_output("gmsh -optimize_netgen -format msh2 -3 mesh.geo -o mesh.msh".split(), stderr=devnull)
 
 
+ def add_polygons(self,argv):
+
+   shapes = get_shape(argv)
+   polygons = [translate_shape(shapes[n],i) for n,i in enumerate(argv['base'])]
+   argv.update({'polygons':np.array(polygons)})
+
+
  def add_symmetry(self,argv):
 
   if argv.setdefault('invariance',False):
       argv['base'] =  np.append(argv['base'],[[0,0]],axis=0)
 
   if argv.setdefault('reflection',False):
-      
+     
+      argv['base']=np.array(argv['base'])
       base = argv['base'].copy()
       tmp = base/2 + np.array([0.25,0.25])
       base = argv['base'].copy()
@@ -614,6 +627,7 @@ class Mesher(object):
   self.common(z=-self.argv['lz']/2)
 
   self.common(z=self.argv['lz']/2)
+
   self.merge() 
   self.points = np.array(self.points)
   self.lines = np.array(self.lines)
@@ -686,6 +700,7 @@ class Mesher(object):
      
      
  def apply_periodic_mesh(self):
+
      self.argv.setdefault('Periodic',[True,True,False])
      #Find periodic surfaces----
      self.periodic = {}
@@ -698,7 +713,6 @@ class Mesher(object):
            pts1,ind1 = self.get_points_from_surface(s1)
            pts2,ind2 = self.get_points_from_surface(s2)
 
-
            for n1,p1 in zip(ind1,pts1):
             for n2,p2 in zip(ind2,pts2):
              if np.linalg.norm(p1-p2-per) < 1e-3:
@@ -730,8 +744,31 @@ class Mesher(object):
    Nh = int(len(self.points)/2)
    for n in range(Nh):
      self.lines.append([n,n+Nh])  
-     
 
+   Ns = int(len(self.surfaces)/2)
+   self.surfaces3D = []
+   for s in range(Ns):
+    tmp2 = [s,s+Ns]  
+
+    for loop in self.surfaces[s]:
+
+     #Create vertical loop    
+     for l in self.loops[loop]:
+      i1 = self.lines[l-1][0];  i2 = self.lines[l-1][1]
+      j1 = i1 + Nh; j2 = i2 + Nh
+      tmp = []
+      tmp.append(self.line_exists_ordered(i1,i2))
+      tmp.append(self.line_exists_ordered(i2,j2))
+      tmp.append(self.line_exists_ordered(j2,j1))
+      tmp.append(self.line_exists_ordered(j1,i1))
+      self.loops.append(tmp)
+      self.surfaces.append([len(self.loops)-1])
+      tmp2.append(len(self.surfaces)-1)
+
+ 
+    self.surfaces3D.append(tmp2)
+
+   '''
    #Create new loops 
    N = int(len(self.loops)/2)
    for ll in range(N):
@@ -748,8 +785,8 @@ class Mesher(object):
   
      self.loops.append(tmp)
      self.surfaces.append([len(self.loops)-1])
-     
- 
+   print(self.loops)
+   '''   
  
  #def apply_periodic_mesh(self):
 
@@ -780,7 +817,7 @@ class Mesher(object):
     store.write(strc)
    
    
-   #Write surface------
+   #Write surface-----
    nll = len(self.loops)
    for s,surface in enumerate(self.surfaces):
     strc = '''Plane Surface(''' +str(s+nll+nl) + ''')= {''' 
@@ -796,6 +833,7 @@ class Mesher(object):
    boundary = list(range(len(self.surfaces)))
    #Apply periodicity
    vec = {}
+
    for key, value in self.periodic.items():
    
      boundary.remove(key)  
@@ -816,7 +854,6 @@ class Mesher(object):
        store.write(',')
      store.write('};\n')
     
-
      value[0] += nl+nll
      tt = key +  nl + nll
      if np.array_equal(np.array([1,0,0]),value[3]) : 
@@ -824,8 +861,8 @@ class Mesher(object):
         vec.setdefault(1,[]).append(tt) 
      
      if np.array_equal(np.array([-1,0,0]),value[3]): 
+        vec.setdefault(1,[]).append(value[0]) 
         vec.setdefault(2,[]).append(tt) 
-        vec.setdefault(1,[]).append(value) 
         
      if np.array_equal(np.array([0,1,0]),value[3]) : 
         vec.setdefault(3,[]).append(value[0]) 
@@ -835,6 +872,7 @@ class Mesher(object):
         vec.setdefault(4,[]).append(value[0]) 
         vec.setdefault(3,[]).append(tt)
         
+
      if np.array_equal(np.array([0,0,1]),value[3]) : 
         vec.setdefault(5,[]).append(value[0]) 
         vec.setdefault(6,[]).append(tt)   
@@ -842,7 +880,6 @@ class Mesher(object):
      if np.array_equal(np.array([0,0,-1]),value[3]) : 
         vec.setdefault(6,[]).append(value[0]) 
         vec.setdefault(5,[]).append(tt) 
-        
    for key, value in vec.items():
     
      strc = r'''Physical Surface("Periodic_''' + str(key) + '''") = {'''
@@ -853,7 +890,7 @@ class Mesher(object):
       else :
        strc += ','
      store.write(strc)
-         
+
    strc = r'''Physical Surface("Boundary") = {'''
    for n,p in enumerate(boundary) :
       strc +=str(p+nl+nll)
@@ -863,24 +900,36 @@ class Mesher(object):
        strc += ','
    store.write(strc)
     
-    
-   if len(self.surfaces) > 0:
+  
+   #this part needs to change---
+
+   index = nll+nl +  len(self.surfaces3D)
+   for k,surfaces in enumerate(self.surfaces3D):
+       
     #------------------
-    ss  = len(self.surfaces)
-    strc = r'''Surface Loop(''' + str(ss+nll+nl) + ') = {'
-    for s in range(len(self.surfaces)) :
-     strc +=str(s+nl+nll)
-     if s == len(self.surfaces)-1:
+    strc = r'''Surface Loop(''' + str(nll+nl + k) + ') = {'
+    for s,surface in enumerate(surfaces) :
+     strc +=str(nll+nl+surface)
+     if s == len(surfaces)-1:
       strc += '};\n'
      else :
       strc += ','
     store.write(strc)
     
-    strc = r'''Volume(0) = {''' + str(ss+nll+nl) + '};\n'
+    strc = r'''Volume(''' + str(index + k) + ''') = {''' + str(nll+nl+k) + '};\n'
     store.write(strc)
-    strc = r'''Physical Volume('Bulk') = {0};'''
-    store.write(strc) 
+  
+   strc = r'''Physical Volume('Bulk') = {'''
+   for k in range(len(self.surfaces3D)) :
+     strc +=str(k+index)
+     if k == len(self.surfaces3D)-1:
+      strc += '};\n'
+     else :
+      strc += ','
+   store.write(strc)
 
+   #---------------------------------------------
+    
    store.close()
 
 
@@ -908,14 +957,19 @@ class Mesher(object):
   return -1
 
 
- def already_included_old(self,all_points,new_point):
+ def already_included_old(self,all_points,new_point,p_list):
+
+
+  dd = 1e-12
+  I = []
+  if len(all_points) > 0:
+   I = np.where(np.linalg.norm(np.array(new_point)-np.array(all_points),axis=1)<dd)[0]
+  if len(I) > 0: 
+     return I[0]
+  else: 
+     return -1
+
 
-  dd = 1e-12   
-  for n,p in enumerate(all_points):
-   d = np.linalg.norm(np.array(p)-np.array(new_point))
-   if d < dd:
-    return n
-  return -1
 
  # return loop
  def line_exists_ordered_old(self,l,lines):
@@ -928,17 +982,25 @@ class Mesher(object):
 
  def create_line_list(self,pp,points,lines,store):
 
+   #Eliminate unncessesary points--
+
+
+   #------------------------------
+
    p_list = []
    for p in pp:
-    tmp = self.already_included_old(points,p)
-    if tmp == -1:
-     points.append(p)
-     #store.write( 'Point('+str(len(points)-1) +') = {' + str(p[0]) +','+ str(p[1])+',0,h};\n')
-     store.write( 'Point('+str(len(points)-1) +') = {' + str(p[0]/self.lx) +'*lx,'+ str(p[1]/self.ly)+'*ly,0,h};\n')
-     p_list.append(len(points)-1)
+    tmp = self.already_included_old(points,p,p_list)
+    if tmp == -1:    
+      points.append(p)
+      p_list.append(len(points)-1)
     else:
-     p_list.append(tmp)
+      p_list.append(tmp)
 
+   for k,p in enumerate(p_list):
+     point = points[-len(p_list)+k] 
+     store.write( 'Point('+str(len(points)-len(p_list)+k) +') = {' + str(point[0]/self.lx) +'*lx,'+ str(point[1]/self.ly)+'*ly,0,h};\n')
+
+   
 
    line_list = []
    for l in range(len(p_list)):
@@ -994,28 +1056,24 @@ class Mesher(object):
   Frame = Polygon(generate_frame(**self.argv))
   polygons = self.argv['polygons']
 
-  polypores = []
-  for poly in polygons:
-   polypores.append(Polygon(poly))
-
+  polypores = [ Polygon(poly)  for poly in polygons]
   
   MP = MultiPolygon(polypores)
   bulk = Frame.difference(cascaded_union(MP))
   if not (isinstance(bulk, shapely.geometry.multipolygon.MultiPolygon)):
    bulk = [bulk]
-  
-  loop_start = len(self.loops)  
+
   for region in bulk:
+   loop_start = len(self.loops)  
    pp = list(region.exterior.coords)[:-1]
-   self.create_lines_and_loop_from_points(pp,z)
-   
+   self.create_lines_and_loop_from_points(pp,z) 
    #From pores 
    for interior in region.interiors:
     pp = list(interior.coords)[:-1]
     self.create_lines_and_loop_from_points(pp,z)
-  loop_end = len(self.loops)  
-  
-  self.surfaces.append(range(loop_start,loop_end))
+   loop_end = len(self.loops) 
+
+   self.surfaces.append(range(loop_start,loop_end))
 
     
 
